@@ -19,7 +19,7 @@ typedef enum {
 	R_INSTALL_ERROR_DOWNLOAD_MF,
 	R_INSTALL_ERROR_HANDLER,
 	R_INSTALL_ERROR_NO_SUPPORTED,
-	R_INSTALL_ERROR_MOUNTED
+	R_INSTALL_ERROR_MOUNTED,
 } RInstallError;
 
 typedef struct {
@@ -29,6 +29,8 @@ typedef struct {
 	GMutex status_mutex;
 	GQueue status_messages;
 	gint status_result;
+	/* install options */
+	gboolean ignore_compatible;
 } RaucInstallArgs;
 
 /**
