@@ -2,8 +2,6 @@
 
 #include <glib.h>
 
-#include <checksum.h>
-
 #if ENABLE_NETWORK
 /**
  * Network initalization routine.
@@ -22,7 +20,4 @@ static inline gboolean network_init(GError **error)
 }
 #endif
 
-gboolean download_file(const gchar *target, const gchar *url, gsize limit, GError **error);
-gboolean download_file_checksum(const gchar *target, const gchar *url,
-		const RaucChecksum *checksum);
-gboolean download_mem(GBytes **data, const gchar *url, gsize limit, GError **error);
+gboolean download_file(const gchar *target, const gchar *url, goffset limit, GError **error);
