@@ -21,12 +21,12 @@ typedef enum {
 	R_SIGNATURE_ERROR_VERIFY_CERT,
 	R_SIGNATURE_ERROR_GET_SIGNER,
 	R_SIGNATURE_ERROR_NUM_SIGNER,
-
 	R_SIGNATURE_ERROR_X509_NEW,
 	R_SIGNATURE_ERROR_X509_LOOKUP,
+	R_SIGNATURE_ERROR_X509_PURPOSE,
 	R_SIGNATURE_ERROR_CA_LOAD,
 	R_SIGNATURE_ERROR_PARSE,
-	R_SIGNATURE_ERROR_INVALID
+	R_SIGNATURE_ERROR_INVALID,
 } RSignatureError;
 
 /**
@@ -40,6 +40,20 @@ typedef enum {
  */
 gboolean signature_init(GError **error);
 
+/**
+ * Prepare an OpenSSL X509_STORE for signature verification.
+ *
+ * When capath or cadir are NULL, this uses the paths and options configured in
+ * the [keyring] section in system.conf or the commandline by default.
+ *
+ * @param capath optional ca file path
+ * @param cadir optional ca directory path
+ * @param error return location for a GError, or NULL
+ *
+ * @return X509_STORE, NULL if failed
+ */
+X509_STORE* setup_x509_store(const gchar *capath, const gchar *cadir, GError **error);
+
 /**
  * Sign content with provided certificate and private key
  *
@@ -93,7 +107,7 @@ gboolean cms_verify(GBytes *content, GBytes *sig, X509_STORE *store, CMS_Content
  *
  * @return TRUE if succeeded, FALSE if failed
  */
-gboolean cms_verify_file(const gchar *filename, GBytes *sig, gsize limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error);
+gboolean cms_verify_file(const gchar *filename, GBytes *sig, goffset limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error);
 
 /**
  * Calculates hash for certificate pubkey info.
