@@ -21,6 +21,7 @@ static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, GEr
 	g_autoptr(GSubprocess) sproc = NULL;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(7, g_free);
 
 	r_context_begin_step("mksquashfs", "Creating squashfs", 0);
 
@@ -29,16 +30,35 @@ static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, GEr
 		goto out;
 	}
 
-	sproc = r_subprocess_new(G_SUBPROCESS_FLAGS_STDOUT_SILENCE,
-			&ierror, "mksquashfs",
-			contentdir,
-			bundlename,
-			"-all-root",
-			"-noappend",
-			"-no-progress",
-			"-no-xattrs",
-			NULL);
+	g_ptr_array_add(args, g_strdup("mksquashfs"));
+	g_ptr_array_add(args, g_strdup(contentdir));
+	g_ptr_array_add(args, g_strdup(bundlename));
+	g_ptr_array_add(args, g_strdup("-all-root"));
+	g_ptr_array_add(args, g_strdup("-noappend"));
+	g_ptr_array_add(args, g_strdup("-no-progress"));
+	g_ptr_array_add(args, g_strdup("-no-xattrs"));
+
+
+	if (r_context()->mksquashfs_args != NULL) {
+		g_auto(GStrv) mksquashfs_argvp = NULL;
+		res = g_shell_parse_argv(r_context()->mksquashfs_args, NULL, &mksquashfs_argvp, &ierror);
+		if (!res) {
+			g_propagate_prefixed_error(
+					error,
+					ierror,
+					"Failed to parse mksquashfs extra args: ");
+			goto out;
+		}
+		for (gchar **mksquashfs_args = mksquashfs_argvp; *mksquashfs_args != NULL; mksquashfs_args++) {
+			g_ptr_array_add(args, g_strdup(*mksquashfs_args));
+		}
+	}
+	g_ptr_array_add(args, NULL);
+
+	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_STDOUT_SILENCE,
+			&ierror);
 	if (sproc == NULL) {
+		res = FALSE;
 		g_propagate_prefixed_error(
 				error,
 				ierror,
@@ -139,6 +159,21 @@ static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath,
 		g_ptr_array_add(iargs, g_strdup("--store"));
 		g_ptr_array_add(iargs, g_strdup(store));
 	}
+
+	if (r_context()->casync_args != NULL) {
+		g_auto(GStrv) casync_argvp = NULL;
+		res = g_shell_parse_argv(r_context()->casync_args, NULL, &casync_argvp, &ierror);
+		if (!res) {
+			g_propagate_prefixed_error(
+					error,
+					ierror,
+					"Failed to parse casync extra args: ");
+			goto out;
+		}
+		for (gchar **casync_args = casync_argvp; *casync_args != NULL; casync_args++) {
+			g_ptr_array_add(iargs, g_strdup(*casync_args));
+		}
+	}
 	g_ptr_array_add(iargs, NULL);
 
 	/* Outer process calll */
@@ -154,6 +189,7 @@ static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath,
 				error,
 				ierror,
 				"Failed to start casync: ");
+		res = FALSE;
 		goto out;
 	}
 
@@ -186,6 +222,21 @@ static gboolean casync_make_blob(const gchar *idxpath, const gchar *contentpath,
 		g_ptr_array_add(args, g_strdup("--store"));
 		g_ptr_array_add(args, g_strdup(store));
 	}
+
+	if (r_context()->casync_args != NULL) {
+		g_auto(GStrv) casync_argvp = NULL;
+		res = g_shell_parse_argv(r_context()->casync_args, NULL, &casync_argvp, &ierror);
+		if (!res) {
+			g_propagate_prefixed_error(
+					error,
+					ierror,
+					"Failed to parse casync extra args: ");
+			goto out;
+		}
+		for (gchar **casync_args = casync_argvp; *casync_args != NULL; casync_args++) {
+			g_ptr_array_add(args, g_strdup(*casync_args));
+		}
+	}
 	g_ptr_array_add(args, NULL);
 
 	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_STDOUT_SILENCE, &ierror);
@@ -194,6 +245,7 @@ static gboolean casync_make_blob(const gchar *idxpath, const gchar *contentpath,
 				error,
 				ierror,
 				"Failed to start casync: ");
+		res = FALSE;
 		goto out;
 	}
 
@@ -391,16 +443,19 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 	res = sign_bundle(bundlename, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
-		g_remove(bundlename);
 		goto out;
 	}
 
 	res = TRUE;
 out:
+	/* Remove output file on error */
+	if (!res && g_file_test(bundlename, G_FILE_TEST_IS_REGULAR))
+		if (g_remove(bundlename) != 0)
+			g_warning("failed to remove %s", bundlename);
 	return res;
 }
 
-static gboolean truncate_bundle(const gchar *inpath, const gchar *outpath, gsize size, GError **error)
+static gboolean truncate_bundle(const gchar *inpath, const gchar *outpath, goffset size, GError **error)
 {
 	g_autoptr(GFile) infile = NULL;
 	g_autoptr(GFile) outfile = NULL;
@@ -410,6 +465,12 @@ static gboolean truncate_bundle(const gchar *inpath, const gchar *outpath, gsize
 	gboolean res = FALSE;
 	gssize ssize;
 
+	if (g_file_test(outpath, G_FILE_TEST_EXISTS)) {
+		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_EXIST, "bundle %s already exists", outpath);
+		res = FALSE;
+		goto out;
+	}
+
 	infile = g_file_new_for_path(inpath);
 	outfile = g_file_new_for_path(outpath);
 
@@ -471,12 +532,15 @@ gboolean resign_bundle(RaucBundle *bundle, const gchar *outpath, GError **error)
 	res = sign_bundle(outpath, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
-		g_remove(outpath);
 		goto out;
 	}
 
 	res = TRUE;
 out:
+	/* Remove output file on error */
+	if (!res && g_file_test(outpath, G_FILE_TEST_IS_REGULAR))
+		if (g_remove(outpath) != 0)
+			g_warning("failed to remove %s", outpath);
 	return res;
 }
 
@@ -498,16 +562,18 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 	g_autofree gchar *contentdir = NULL;
 	g_autofree gchar *mfpath = NULL;
 	g_autofree gchar *storepath = NULL;
-	gchar *basepath = NULL;
 	g_autoptr(RaucManifest) manifest = NULL;
 
 	g_return_val_if_fail(bundle, FALSE);
 	g_return_val_if_fail(outbundle, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	basepath = g_strndup(outbundle, strlen(outbundle) - 6);
-	storepath = g_strconcat(basepath, ".castr", NULL);
-	g_free(basepath);
+	if (g_str_has_suffix(outbundle, ".raucb")) {
+		g_autofree gchar *basepath = g_strndup(outbundle, strlen(outbundle) - 6);
+		storepath = g_strconcat(basepath, ".castr", NULL);
+	} else {
+		storepath = g_strconcat(outbundle, ".castr", NULL);
+	}
 
 	/* Assure bundle destination path doe not already exist */
 	if (g_file_test(outbundle, G_FILE_TEST_EXISTS)) {
@@ -567,7 +633,6 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 				goto out;
 			}
 		} else {
-
 			idxfile = g_strconcat(image->filename, ".caibx", NULL);
 			idxpath = g_build_filename(contentdir, idxfile, NULL);
 
@@ -587,7 +652,7 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 
 		/* Remove original file */
 		if (g_remove(imgpath) != 0) {
-			g_warning("Failed removing %s", imgpath);
+			g_warning("failed to remove %s", imgpath);
 		}
 	}
 
@@ -682,15 +747,15 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 	if (r_context()->config->store_path) {
 		ibundle->storepath = r_context()->config->store_path;
 	} else {
-		gchar *strprfx;
+		gchar *path = ibundle->origpath ?: ibundle->path;
 
-		if (ibundle->origpath)
-			strprfx = g_strndup(ibundle->origpath, strlen(ibundle->origpath) - 6);
-		else
-			strprfx = g_strndup(ibundle->path, strlen(ibundle->path) - 6);
-		ibundle->storepath = g_strconcat(strprfx, ".castr", NULL);
-
-		g_free(strprfx);
+		if (g_str_has_suffix(path, ".raucb")) {
+			g_autofree gchar *strprfx;
+			strprfx = g_strndup(path, strlen(path) - 6);
+			ibundle->storepath = g_strconcat(strprfx, ".castr", NULL);
+		} else {
+			ibundle->storepath = g_strconcat(path, ".castr", NULL);
+		}
 	}
 
 	if (verify && !r_context()->config->keyring_path && !r_context()->config->keyring_directory) {
@@ -789,26 +854,11 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, gboolean ver
 	}
 
 	if (verify) {
-		const gchar *load_capath = r_context()->config->keyring_path;
-		const gchar *load_cadir = r_context()->config->keyring_directory;
 		CMS_ContentInfo *cms = NULL;
-		X509_STORE *store = NULL;
-
-		if (!(store = X509_STORE_new())) {
-			g_set_error_literal(
-					error,
-					R_SIGNATURE_ERROR,
-					R_SIGNATURE_ERROR_X509_NEW,
-					"failed to allocate new X509 store");
-			goto out;
-		}
-		if (!X509_STORE_load_locations(store, load_capath, load_cadir)) {
-			g_set_error(
-					error,
-					R_SIGNATURE_ERROR,
-					R_SIGNATURE_ERROR_CA_LOAD,
-					"failed to load CA file '%s' and/or directory '%s'", load_capath, load_cadir);
-
+		X509_STORE *store = setup_x509_store(NULL, NULL, &ierror);
+		if (!store) {
+			g_propagate_error(error, ierror);
+			res = FALSE;
 			goto out;
 		}
 
@@ -944,10 +994,10 @@ void free_bundle(RaucBundle *bundle)
 {
 	g_return_if_fail(bundle);
 
-	/* In case of a temporary donwload artifact, remove it. */
+	/* In case of a temporary download artifact, remove it. */
 	if (bundle->origpath)
-		if (g_remove(bundle->path) == -1) {
-			g_warning("Failed removing download artifact %s: %s\n", bundle->path, g_strerror(errno));
+		if (g_remove(bundle->path) != 0) {
+			g_warning("failed to remove download artifact %s: %s\n", bundle->path, g_strerror(errno));
 		}
 
 	g_free(bundle->path);
