@@ -228,6 +228,18 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 	c->keyring_directory = resolve_path(filename,
 			key_file_consume_string(key_file, "keyring", "directory", NULL));
 
+	c->keyring_check_crl = g_key_file_get_boolean(key_file, "keyring", "check-crl", &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND) ||
+	    g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
+		c->keyring_check_crl = FALSE;
+		g_clear_error(&ierror);
+	} else if (ierror) {
+		g_propagate_error(error, ierror);
+		res = FALSE;
+		goto free;
+	}
+	g_key_file_remove_key(key_file, "keyring", "check-crl", NULL);
+
 	c->use_bundle_signing_time = g_key_file_get_boolean(key_file, "keyring", "use-bundle-signing-time", &ierror);
 	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND) ||
 	    g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
@@ -240,6 +252,17 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 	}
 	g_key_file_remove_key(key_file, "keyring", "use-bundle-signing-time", NULL);
 
+	c->keyring_check_purpose = key_file_consume_string(key_file, "keyring", "check-purpose", &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND) ||
+	    g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
+		c->keyring_check_purpose = NULL;
+		g_clear_error(&ierror);
+	} else if (ierror) {
+		g_propagate_error(error, ierror);
+		res = FALSE;
+		goto free;
+	}
+
 	if (!check_remaining_keys(key_file, "keyring", &ierror)) {
 		g_propagate_error(error, ierror);
 		res = FALSE;
@@ -322,8 +345,6 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			g_free(value);
 
 			slot->description = key_file_consume_string(key_file, groups[i], "description", NULL);
-			if (!slot->description)
-				slot->description = g_strdup("");
 
 			slot->sclass = g_intern_string(groupsplit[1]);
 
@@ -344,6 +365,16 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			value = key_file_consume_string(key_file, groups[i], "bootname", NULL);
 			slot->bootname = value;
 
+			slot->allow_mounted = g_key_file_get_boolean(key_file, groups[i], "allow-mounted", &ierror);
+			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+				slot->allow_mounted = FALSE;
+				g_clear_error(&ierror);
+			} else if (ierror) {
+				g_propagate_error(error, ierror);
+				res = FALSE;
+				goto free;
+			}
+
 			slot->readonly = g_key_file_get_boolean(key_file, groups[i], "readonly", &ierror);
 			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
 				slot->readonly = FALSE;
@@ -367,20 +398,17 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 					if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
 						slot->install_same = TRUE;
 						g_clear_error(&ierror);
-					}
-					else if (ierror) {
+					} else if (ierror) {
 						g_propagate_error(error, ierror);
 						res = FALSE;
 						goto free;
 					}
-				}
-				else if (ierror) {
+				} else if (ierror) {
 					g_propagate_error(error, ierror);
 					res = FALSE;
 					goto free;
 				}
-			}
-			else if (ierror) {
+			} else if (ierror) {
 				g_propagate_error(error, ierror);
 				res = FALSE;
 				goto free;
@@ -395,8 +423,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
 				slot->resize = FALSE;
 				g_clear_error(&ierror);
-			}
-			else if (ierror) {
+			} else if (ierror) {
 				g_propagate_error(error, ierror);
 				res = FALSE;
 				goto free;
@@ -489,6 +516,15 @@ RaucSlot *find_config_slot_by_device(RaucConfig *config, const gchar *device)
 	return r_slot_find_by_device(config->slots, device);
 }
 
+RaucSlot *find_config_slot_by_name(RaucConfig *config, const gchar *name)
+{
+	g_return_val_if_fail(config, NULL);
+	g_return_val_if_fail(config->slots, NULL);
+	g_return_val_if_fail(name, NULL);
+
+	return g_hash_table_lookup(config->slots, name);
+}
+
 void free_config(RaucConfig *config)
 {
 	g_return_if_fail(config);
@@ -502,6 +538,7 @@ void free_config(RaucConfig *config)
 	g_free(config->statusfile_path);
 	g_free(config->keyring_path);
 	g_free(config->keyring_directory);
+	g_free(config->keyring_check_purpose);
 	g_free(config->autoinstall_path);
 	g_free(config->systeminfo_handler);
 	g_free(config->preinstall_handler);
