@@ -126,8 +126,7 @@ gboolean determine_slot_states(GError **error)
 	slotlist = g_hash_table_get_keys(r_context()->config->slots);
 
 	for (GList *l = slotlist; l != NULL; l = l->next) {
-		gchar buf[PATH_MAX + 1];
-		gchar *realdev = NULL;
+		g_autofree gchar *realdev = NULL;
 
 		RaucSlot *s = (RaucSlot*) g_hash_table_lookup(r_context()->config->slots, l->data);
 
@@ -141,10 +140,10 @@ gboolean determine_slot_states(GError **error)
 			break;
 		}
 
-		realdev = realpath(s->device, buf);
+		realdev = r_realpath(s->device);
 		if (realdev == NULL) {
 			g_message("Failed to resolve realpath for '%s'", s->device);
-			realdev = s->device;
+			realdev = g_strdup(s->device);
 		}
 
 		if (g_strcmp0(realdev, r_context()->bootslot) == 0) {
@@ -182,6 +181,12 @@ gboolean determine_slot_states(GError **error)
 			goto out;
 		}
 
+		if (g_strcmp0(r_context()->bootslot, "_external_") == 0) {
+			g_message("Detected explicit external boot, ignoring missing active slot");
+			res = TRUE;
+			goto out;
+		}
+
 		g_set_error_literal(
 				error,
 				R_SLOT_ERROR,
@@ -249,7 +254,7 @@ static gchar** get_all_manifest_slot_classes(const RaucManifest *manifest)
  *
  * @return pointer to appropriate slot in system slot list
  */
-static RaucSlot *select_inactive_slot_class_member(gchar *rootclass)
+static RaucSlot *select_inactive_slot_class_member(const gchar *rootclass)
 {
 	RaucSlot *iterslot;
 	GHashTableIter iter;
@@ -441,9 +446,9 @@ static void parse_handler_output(gchar* line)
 	}
 }
 
-static gboolean verify_compatible(RaucManifest *manifest)
+static gboolean verify_compatible(RaucInstallArgs *args, RaucManifest *manifest)
 {
-	if (r_context()->ignore_compatible) {
+	if (args->ignore_compatible) {
 		return TRUE;
 	} else if (g_strcmp0(r_context()->config->system_compatible,
 			manifest->update_compatible) == 0) {
@@ -466,8 +471,10 @@ static void prepare_environment(GSubprocessLauncher *launcher, gchar *update_sou
 
 	g_subprocess_launcher_setenv(launcher, "RAUC_SYSTEM_CONFIG", r_context()->configpath, TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_CURRENT_BOOTNAME", r_context()->bootslot, TRUE);
-	g_subprocess_launcher_setenv(launcher, "RAUC_UPDATE_SOURCE", update_source, TRUE);
 	g_subprocess_launcher_setenv(launcher, "RAUC_MOUNT_PREFIX", r_context()->config->mount_prefix, TRUE);
+	g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_MOUNT_POINT", update_source, TRUE);
+	/* Deprecated, included for backwards compatibility: */
+	g_subprocess_launcher_setenv(launcher, "RAUC_UPDATE_SOURCE", update_source, TRUE);
 
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
@@ -684,7 +691,7 @@ static gboolean launch_and_wait_custom_handler(RaucInstallArgs *args, gchar* bun
 			res = FALSE;
 			goto out;
 		}
-	} else if (!verify_compatible(manifest)) {
+	} else if (!verify_compatible(args, manifest)) {
 		g_set_error_literal(error, R_INSTALL_ERROR, R_INSTALL_ERROR_COMPAT_MISMATCH,
 				"Compatible mismatch");
 		res = FALSE;
@@ -700,6 +707,27 @@ out:
 	return res;
 }
 
+static gboolean pre_install_check_slot_mount_status(RaucSlot *slot, RaucImage *mfimage, GError **error)
+{
+	/* OK if the slot is not mounted at all. */
+	if (!(slot->mount_point || slot->ext_mount_point))
+		return TRUE;
+
+	/* OK if the configuration says it may already be mounted and the
+	 * bundle has a custom install hook. */
+	if (slot->allow_mounted && mfimage->hooks.install)
+		return TRUE;
+
+	if (slot->allow_mounted)
+		g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_MOUNTED,
+				"Mounted device '%s' may only be updated by a custom install hook", slot->device);
+	else
+		g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_MOUNTED,
+				"Destination device '%s' already mounted", slot->device);
+
+	return FALSE;
+}
+
 static gboolean pre_install_checks(gchar* bundledir, GList *install_images, GHashTable *target_group, GError **error)
 {
 	for (GList *l = install_images; l != NULL; l = l->next) {
@@ -725,9 +753,8 @@ static gboolean pre_install_checks(gchar* bundledir, GList *install_images, GHas
 			return FALSE;
 		}
 
-		if (dest_slot->mount_point || dest_slot->ext_mount_point) {
-			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_MOUNTED,
-					"Destination device '%s' already mounted", dest_slot->device);
+		if (!pre_install_check_slot_mount_status(dest_slot, mfimage, error)) {
+			/* error is already set */
 			return FALSE;
 		}
 	}
@@ -741,7 +768,6 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	GError *ierror = NULL;
 	gboolean res = FALSE;
 	GList *install_images = NULL;
-	RaucImage *mfimage;
 
 	install_images = get_install_images(manifest, target_group, &ierror);
 	if (install_images == NULL) {
@@ -767,7 +793,7 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 			res = FALSE;
 			goto early_out;
 		}
-	} else if (!verify_compatible(manifest)) {
+	} else if (!verify_compatible(args, manifest)) {
 		res = FALSE;
 		g_set_error_literal(error, R_INSTALL_ERROR, R_INSTALL_ERROR_COMPAT_MISMATCH,
 				"Compatible mismatch");
@@ -807,6 +833,7 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	r_context_begin_step("update_slots", "Updating slots", g_list_length(install_images) * 2);
 	install_args_update(args, "Updating slots...");
 	for (GList *l = install_images; l != NULL; l = l->next) {
+		RaucImage *mfimage;
 		RaucSlot *dest_slot;
 		img_to_slot_handler update_handler = NULL;
 		RaucSlotStatus *slot_state = NULL;
