@@ -6,7 +6,9 @@
 #include <json-glib/json-glib.h>
 #include <json-glib/json-gobject.h>
 #endif
+#include <locale.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
 
@@ -26,10 +28,14 @@ GMainLoop *r_loop = NULL;
 int r_exit_status = 0;
 
 gboolean install_ignore_compatible, install_progressbar = FALSE;
-gboolean info_noverify, info_dumpcert = FALSE;
+gboolean verification_disabled = FALSE;
+gboolean info_dumpcert = FALSE;
 gboolean status_detailed = FALSE;
 gchar *output_format = NULL;
 gchar *signing_keyring = NULL;
+gchar *mksquashfs_args = NULL;
+gchar *casync_args = NULL;
+gboolean utf8_supported = FALSE;
 
 static gchar* make_progress_line(gint percentage)
 {
@@ -219,10 +225,14 @@ static gboolean install_start(int argc, char **argv)
 	args->cleanup = install_cleanup;
 	args->status_result = 2;
 
-	r_context_conf()->ignore_compatible = install_ignore_compatible;
+	args->ignore_compatible = install_ignore_compatible;
 
 	r_loop = g_main_loop_new(NULL, FALSE);
 	if (ENABLE_SERVICE) {
+		g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
+
+		g_variant_dict_insert(&dict, "ignore-compatible", "b", args->ignore_compatible);
+
 		installer = r_installer_proxy_new_for_bus_sync(bus_type,
 				G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES,
 				"de.pengutronix.rauc", "/", NULL, &error);
@@ -242,7 +252,11 @@ static gboolean install_start(int argc, char **argv)
 			goto out_loop;
 		}
 		g_debug("Trying to contact rauc service");
-		if (!r_installer_call_install_sync(installer, args->name, NULL,
+		if (!r_installer_call_install_bundle_sync(
+				installer,
+				args->name,
+				g_variant_dict_end(&dict), /* floating, no unref needed */
+				NULL,
 				&error)) {
 			g_printerr("Failed %s\n", error->message);
 			g_error_free(error);
@@ -282,6 +296,7 @@ out:
 	return TRUE;
 }
 
+G_GNUC_UNUSED
 static gboolean bundle_start(int argc, char **argv)
 {
 	GError *ierror = NULL;
@@ -445,6 +460,7 @@ out:
 	return TRUE;
 }
 
+G_GNUC_UNUSED
 static gboolean resign_start(int argc, char **argv)
 {
 	g_autoptr(RaucBundle) bundle = NULL;
@@ -477,7 +493,7 @@ static gboolean resign_start(int argc, char **argv)
 		goto out;
 	}
 
-	if (!check_bundle(argv[2], &bundle, TRUE, &ierror)) {
+	if (!check_bundle(argv[2], &bundle, !verification_disabled, &ierror)) {
 		g_printerr("%s\n", ierror->message);
 		g_clear_error(&ierror);
 		r_exit_status = 1;
@@ -540,6 +556,7 @@ out:
 	return TRUE;
 }
 
+G_GNUC_UNUSED
 static gboolean convert_start(int argc, char **argv)
 {
 	RaucBundle *bundle = NULL;
@@ -594,47 +611,6 @@ out:
 	return TRUE;
 }
 
-static gboolean checksum_start(int argc, char **argv)
-{
-	GError *error = NULL;
-	gboolean sign = FALSE;
-
-	g_debug("checksum start");
-
-	if (r_context()->certpath != NULL &&
-	    r_context()->keypath != NULL) {
-		sign = TRUE;
-	} else if (r_context()->certpath != NULL ||
-	           r_context()->keypath != NULL) {
-		g_printerr("Either both or none of cert and key files must be provided\n");
-		r_exit_status = 1;
-		goto out;
-	}
-
-	if (argc < 3) {
-		g_printerr("A directory name must be provided\n");
-		r_exit_status = 1;
-		goto out;
-	}
-
-	if (argc > 3) {
-		g_printerr("Excess argument: %s\n", argv[3]);
-		r_exit_status = 1;
-		goto out;
-	}
-
-	g_message("updating checksums for: %s", argv[2]);
-
-	if (!update_manifest(argv[2], sign, &error)) {
-		g_printerr("Failed to update manifest: %s\n", error->message);
-		g_clear_error(&error);
-		r_exit_status = 1;
-	}
-
-out:
-	return TRUE;
-}
-
 /* Takes a shell variable and its desired argument as input and appends it to
  * the provided text with taking care of correct shell quoting */
 static void formatter_shell_append(GString* text, const gchar* varname, const gchar* argument)
@@ -683,7 +659,7 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 		formatter_shell_append_n(text, "RAUC_IMAGE_CLASS", cnt, img->slotclass);
 		formatter_shell_append_n(text, "RAUC_IMAGE_VARIANT", cnt, img->variant);
 		formatter_shell_append_n(text, "RAUC_IMAGE_DIGEST", cnt, img->checksum.digest);
-		g_string_append_printf(text, "RAUC_IMAGE_SIZE_%d=%"G_GSIZE_FORMAT "\n", cnt, img->checksum.size);
+		g_string_append_printf(text, "RAUC_IMAGE_SIZE_%d=%"G_GOFFSET_FORMAT "\n", cnt, img->checksum.size);
 
 		hooks = g_ptr_array_new();
 		if (img->hooks.pre_install == TRUE) {
@@ -712,7 +688,7 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 		g_string_append_printf(text, "RAUC_FILE_CLASS_%d=%s\n", cnt, file->slotclass);
 		g_string_append_printf(text, "RAUC_FILE_DEST_%d=%s\n", cnt, file->destname);
 		g_string_append_printf(text, "RAUC_FILE_DIGEST_%d=%s\n", cnt, file->checksum.digest);
-		g_string_append_printf(text, "RAUC_FILE_SIZE_%d=%"G_GSIZE_FORMAT "\n", cnt, file->checksum.size);
+		g_string_append_printf(text, "RAUC_FILE_SIZE_%d=%"G_GOFFSET_FORMAT "\n", cnt, file->checksum.size);
 		cnt++;
 	}
 
@@ -753,7 +729,7 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		if (img->variant)
 			g_string_append_printf(text, "\tVariant:   %s\n", img->variant);
 		g_string_append_printf(text, "\tChecksum:  %s\n", img->checksum.digest);
-		g_string_append_printf(text, "\tSize:      %"G_GSIZE_FORMAT "\n", img->checksum.size);
+		g_string_append_printf(text, "\tSize:      %"G_GOFFSET_FORMAT "\n", img->checksum.size);
 
 		hooks = g_ptr_array_new();
 		if (img->hooks.pre_install == TRUE) {
@@ -785,7 +761,7 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		g_string_append_printf(text, "\tSlotclass: %s\n", file->slotclass);
 		g_string_append_printf(text, "\tDest:      %s\n", file->destname);
 		g_string_append_printf(text, "\tChecksum:  %s\n", file->checksum.digest);
-		g_string_append_printf(text, "\tSize:      %"G_GSIZE_FORMAT "\n", file->checksum.size);
+		g_string_append_printf(text, "\tSize:      %"G_GOFFSET_FORMAT "\n", file->checksum.size);
 		cnt++;
 	}
 
@@ -852,7 +828,6 @@ static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 		json_builder_end_array(builder);
 		json_builder_end_object(builder);
 		json_builder_end_object(builder);
-
 	}
 
 	json_builder_end_array(builder);
@@ -933,7 +908,7 @@ static gboolean info_start(int argc, char **argv)
 		goto out;
 	g_debug("input bundle: %s", bundlelocation);
 
-	res = check_bundle(bundlelocation, &bundle, !info_noverify, &error);
+	res = check_bundle(bundlelocation, &bundle, !verification_disabled, &error);
 	if (!res) {
 		g_printerr("%s\n", error->message);
 		g_clear_error(&error);
@@ -973,7 +948,6 @@ static gboolean info_start(int argc, char **argv)
 		text = print_cert_chain(bundle->verified_chain);
 		g_print("%s\n", text);
 		g_free(text);
-
 	}
 
 out:
@@ -1008,23 +982,48 @@ static void free_status_print(RaucStatusPrint *status)
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(RaucStatusPrint, free_status_print);
 
+/* Definition list for terminal colors */
+#define KNRM  "\x1B[0m"
+#define KRED  "\x1B[31m"
+#define KGRN  "\x1B[32m"
+#define KYEL  "\x1B[33m"
+#define KBLU  "\x1B[34m"
+#define KMAG  "\x1B[35m"
+#define KCYN  "\x1B[36m"
+#define KWHT  "\x1B[37m"
+#define KBLD  "\x1B[1m"
+
 static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint *status)
 {
 	RaucSlotStatus *slot_state = slot->status;
 
-	g_string_append_printf(text, "\n  %s: class=%s, device=%s, type=%s, bootname=%s\n",
-			slot->name, slot->sclass, slot->device, slot->type, slot->bootname);
-	g_string_append_printf(text, "      state=%s, description=%s", r_slot_slotstate_to_str(slot->state), slot->description);
-	if (slot->parent)
-		g_string_append_printf(text, ", parent=%s", slot->parent->name);
-	else
-		g_string_append(text, ", parent=(none)");
-	if (slot->mount_point || slot->ext_mount_point)
-		g_string_append_printf(text, ", mountpoint=%s", slot->mount_point ? slot->mount_point : slot->ext_mount_point);
-	else
-		g_string_append(text, ", mountpoint=(none)");
+	/* bootname slots get an empty or full dot depending on whether they
+	 * are primary or not */
+	if (slot->bootname) {
+		if (utf8_supported)
+			g_string_append_unichar(text, slot == status->primary ? 0x23FA :  0x25CB);
+		else
+			g_string_append(text, slot == status->primary ? KBLD "x"KNRM :  "o");
+	} else {
+		g_string_append_c(text, ' ');
+	}
+	g_string_append_printf(text, "%s%s%s"KBLD "[%s]"KNRM " (%s, %s, %s%s"KNRM ")",
+			slot->parent ? "   " : " ",
+			slot->state & ST_ACTIVE ? KGRN : "",
+			slot->state == ST_BOOTED ? KGRN : "",
+			slot->name,
+			slot->device,
+			slot->type,
+			slot->state == ST_BOOTED ? KBLD : "",
+			r_slot_slotstate_to_str(slot->state));
 	if (slot->bootname)
-		g_string_append_printf(text, "\n      boot status=%s", slot->boot_good ? "good" : "bad");
+		g_string_append_printf(text, "\n\tbootname: "KBLU "%s"KNRM, slot->bootname);
+	if (slot->description)
+		g_string_append_printf(text, "\n\tdescription: %s", slot->description);
+	if (slot->mount_point)
+		g_string_append_printf(text, "\n\tmounted: %s", slot->mount_point);
+	if (slot->bootname)
+		g_string_append_printf(text, "\n\tboot status: %s", slot->boot_good ? KGRN "good"KNRM : KRED "bad"KNRM);
 	if (status_detailed && slot_state) {
 		g_string_append_printf(text, "\n      slot status:");
 		g_string_append_printf(text, "\n          bundle:");
@@ -1038,7 +1037,7 @@ static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint
 		if (slot_state->checksum.digest && slot_state->checksum.type == G_CHECKSUM_SHA256) {
 			g_string_append_printf(text, "\n          checksum:");
 			g_string_append_printf(text, "\n              sha256=%s", slot_state->checksum.digest);
-			g_string_append_printf(text, "\n              size=%"G_GSIZE_FORMAT, slot_state->checksum.size);
+			g_string_append_printf(text, "\n              size=%"G_GOFFSET_FORMAT, slot_state->checksum.size);
 		}
 		if (slot_state->installed_timestamp) {
 			g_string_append_printf(text, "\n          installed:");
@@ -1053,6 +1052,7 @@ static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint
 		if (slot_state->status)
 			g_string_append_printf(text, "\n          status=%s", slot_state->status);
 	}
+	g_string_append_c(text, '\n');
 }
 
 static gchar* r_status_formatter_readable(RaucStatusPrint *status)
@@ -1067,13 +1067,15 @@ static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 	if (!bootedfrom)
 		bootedfrom = r_slot_find_by_bootname(status->slots, status->bootslot);
 
+	g_string_append(text, "=== System Info ===\n");
 	g_string_append_printf(text, "Compatible:  %s\n", status->compatible);
 	g_string_append_printf(text, "Variant:     %s\n", status->variant);
-	g_string_append_printf(text, "Booted from: %s (%s)\n", bootedfrom ? bootedfrom->name : NULL, status->bootslot);
-	g_string_append_printf(text, "Activated:   %s (%s)\n", status->primary ? status->primary->name : NULL, status->primary ? status->primary->bootname : NULL);
+	g_string_append_printf(text, "Booted from: %s (%s)\n\n", bootedfrom ? bootedfrom->name : NULL, status->bootslot);
 
-	g_string_append(text, "slot states:");
+	g_string_append(text, "=== Bootloader ===\n");
+	g_string_append_printf(text, "Activated: %s (%s)\n\n", status->primary ? status->primary->name : NULL, status->primary ? status->primary->bootname : NULL);
 
+	g_string_append(text, "=== Slot States ===\n");
 	slotclasses = r_slot_get_root_classes(status->slots);
 
 	for (gchar **cls = slotclasses; *cls != NULL; cls++) {
@@ -1097,7 +1099,6 @@ static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 
 			g_string_append(text, "\n");
 		}
-
 	}
 
 	return g_string_free(text, FALSE);
@@ -1166,7 +1167,7 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_DESCRIPTION", slotcnt, slot_state->bundle_description);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_BUILD", slotcnt, slot_state->bundle_build);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_CHECKSUM_SHA256", slotcnt, slot_state->checksum.digest);
-			str = g_strdup_printf("%"G_GSIZE_FORMAT, slot_state->checksum.size);
+			str = g_strdup_printf("%"G_GOFFSET_FORMAT, slot_state->checksum.size);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_CHECKSUM_SIZE", slotcnt, str);
 			g_free(str);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_INSTALLED_TIMESTAMP", slotcnt, slot_state->installed_timestamp);
@@ -1366,6 +1367,8 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 			G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
 			"de.pengutronix.rauc", "/", NULL, &ierror);
 	if (proxy == NULL) {
+		if (g_dbus_error_is_remote_error(ierror))
+			g_dbus_error_strip_remote_error(ierror);
 		g_set_error(error,
 				G_IO_ERROR,
 				G_IO_ERROR_FAILED,
@@ -1376,6 +1379,8 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 
 	g_debug("Trying to contact rauc service");
 	if (!r_installer_call_get_slot_status_sync(proxy, &slot_status_array, NULL, &ierror)) {
+		if (g_dbus_error_is_remote_error(ierror))
+			g_dbus_error_strip_remote_error(ierror);
 		g_set_error(error,
 				G_IO_ERROR,
 				G_IO_ERROR_FAILED,
@@ -1501,8 +1506,7 @@ static gboolean retrieve_status_via_dbus(RaucStatusPrint **status_print, GError
 
 	/* Obtain configured slots and their state */
 	if (!retrieve_slot_states_via_dbus(&istatus->slots, &ierror)) {
-		g_propagate_prefixed_error(error, ierror, "rauc status: error retrieving slot status via D-Bus: ");
-		g_error_free(ierror);
+		g_propagate_error(error, ierror);
 		return FALSE;
 	}
 
@@ -1589,11 +1593,10 @@ static gboolean status_start(int argc, char **argv)
 		status_print->variant = r_context()->config->system_variant;
 		status_print->bootslot = r_context()->bootslot;
 		status_print->slots = r_context()->config->slots;
-
 	} else {
 		if (!retrieve_status_via_dbus(&status_print, &ierror)) {
 			message = g_strdup_printf(
-					"rauc status: error retrieving slot status via D-Bus: %s",
+					"error retrieving slot status via D-Bus: %s",
 					ierror->message);
 			g_error_free(ierror);
 			r_exit_status = 1;
@@ -1653,13 +1656,13 @@ static gboolean status_start(int argc, char **argv)
 
 out:
 	if (message)
-		g_message("rauc mark: %s", message);
+		g_message("rauc status: %s", message);
 	g_clear_pointer(&proxy, g_object_unref);
 
 	return TRUE;
 }
 
-#if ENABLE_SERVICE == 1
+G_GNUC_UNUSED
 static gboolean service_start(int argc, char **argv)
 {
 	g_debug("service start");
@@ -1668,7 +1671,6 @@ static gboolean service_start(int argc, char **argv)
 
 	return TRUE;
 }
-#endif
 
 static gboolean unknown_start(int argc, char **argv)
 {
@@ -1684,7 +1686,6 @@ typedef enum  {
 	RESIGN,
 	EXTRACT,
 	CONVERT,
-	CHECKSUM,
 	STATUS,
 	INFO,
 	WRITE_SLOT,
@@ -1701,40 +1702,74 @@ typedef struct {
 	gboolean while_busy;
 } RaucCommand;
 
-GOptionEntry entries_install[] = {
+static GOptionEntry entries_install[] = {
 	{"ignore-compatible", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_compatible, "disable compatible check", NULL},
 	{"progress", '\0', 0, G_OPTION_ARG_NONE, &install_progressbar, "show progress bar", NULL},
 	{0}
 };
 
-GOptionEntry entries_bundle[] = {
+static GOptionEntry entries_bundle[] = {
+	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
+	{"mksquashfs-args", '\0', 0, G_OPTION_ARG_STRING, &mksquashfs_args, "mksquashfs extra args", "ARGS"},
+	{0}
+};
+
+static GOptionEntry entries_resign[] = {
+	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
 	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
 	{0}
 };
 
-GOptionEntry entries_resign[] = {
+static GOptionEntry entries_convert[] = {
 	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
+	{"mksquashfs-args", '\0', 0, G_OPTION_ARG_STRING, &mksquashfs_args, "mksquashfs extra args", "ARGS"},
+	{"casync-args", '\0', 0, G_OPTION_ARG_STRING, &casync_args, "casync extra args", "ARGS"},
 	{0}
 };
 
-GOptionEntry entries_convert[] = {
-	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
-	{0}
-};
-
-GOptionEntry entries_info[] = {
-	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &info_noverify, "disable bundle verification", NULL},
+static GOptionEntry entries_info[] = {
+	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
 	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
 	{"dump-cert", '\0', 0, G_OPTION_ARG_NONE, &info_dumpcert, "dump certificate", NULL},
 	{0}
 };
 
-GOptionEntry entries_status[] = {
+static GOptionEntry entries_status[] = {
 	{"detailed", '\0', 0, G_OPTION_ARG_NONE, &status_detailed, "show more status details", NULL},
 	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
 	{0}
 };
 
+static GOptionGroup *install_group;
+static GOptionGroup *bundle_group;
+static GOptionGroup *resign_group;
+static GOptionGroup *convert_group;
+static GOptionGroup *info_group;
+static GOptionGroup *status_group;
+
+static void create_option_groups(void)
+{
+	install_group = g_option_group_new("install", "Install options:", "help dummy", NULL, NULL);
+	g_option_group_add_entries(install_group, entries_install);
+
+	if (ENABLE_CREATE) {
+		bundle_group  = g_option_group_new("bundle", "Bundle options:", "help dummy", NULL, NULL);
+		g_option_group_add_entries(bundle_group, entries_bundle);
+
+		resign_group  = g_option_group_new("resign", "Resign options:", "help dummy", NULL, NULL);
+		g_option_group_add_entries(resign_group, entries_resign);
+
+		convert_group = g_option_group_new("convert", "Convert options:", "help dummy", NULL, NULL);
+		g_option_group_add_entries(convert_group, entries_convert);
+	}
+
+	info_group    = g_option_group_new("info", "Info options:", "help dummy", NULL, NULL);
+	g_option_group_add_entries(info_group, entries_info);
+
+	status_group  = g_option_group_new("status", "Status options:", "help dummy", NULL, NULL);
+	g_option_group_add_entries(status_group, entries_status);
+}
+
 static void cmdline_handler(int argc, char **argv)
 {
 	gboolean help = FALSE, debug = FALSE, version = FALSE;
@@ -1756,12 +1791,6 @@ static void cmdline_handler(int argc, char **argv)
 		{"help", 'h', 0, G_OPTION_ARG_NONE, &help, NULL, NULL},
 		{0}
 	};
-	GOptionGroup *install_group = g_option_group_new("install", "Install options:", "help dummy", NULL, NULL);
-	GOptionGroup *bundle_group = g_option_group_new("bundle", "Bundle options:", "help dummy", NULL, NULL);
-	GOptionGroup *resign_group = g_option_group_new("resign", "Resign options:", "help dummy", NULL, NULL);
-	GOptionGroup *convert_group = g_option_group_new("convert", "Convert options:", "help dummy", NULL, NULL);
-	GOptionGroup *info_group = g_option_group_new("info", "Info options:", "help dummy", NULL, NULL);
-	GOptionGroup *status_group = g_option_group_new("status", "Status options:", "help dummy", NULL, NULL);
 
 	GError *error = NULL;
 	g_autofree gchar *text = NULL;
@@ -1769,13 +1798,20 @@ static void cmdline_handler(int argc, char **argv)
 	RaucCommand rcommands[] = {
 		{UNKNOWN, "help", "<COMMAND>", "Print help", unknown_start, NULL, TRUE},
 		{INSTALL, "install", "install <BUNDLE>", "Install a bundle", install_start, install_group, FALSE},
+#if ENABLE_CREATE == 1
 		{BUNDLE, "bundle", "bundle <INPUTDIR> <BUNDLENAME>", "Create a bundle from a content directory", bundle_start, bundle_group, FALSE},
 		{RESIGN, "resign", "resign <BUNDLENAME>", "Resign an already signed bundle", resign_start, resign_group, FALSE},
-		{EXTRACT, "extract", "extract <BUNDLENAME> <OUTPUTDIR>", "Extract the bundle content", extract_start, NULL, FALSE},
 		{CONVERT, "convert", "convert <INBUNDLE> <OUTBUNDLE>", "Convert to casync index bundle and store", convert_start, convert_group, FALSE},
-		{CHECKSUM, "checksum", "checksum <DIRECTORY>", "Deprecated", checksum_start, NULL, FALSE},
+#endif
+		{EXTRACT, "extract", "extract <BUNDLENAME> <OUTPUTDIR>", "Extract the bundle content", extract_start, NULL, FALSE},
 		{INFO, "info", "info <FILE>", "Print bundle info", info_start, info_group, FALSE},
-		{STATUS, "status", "status", "Show system status", status_start, status_group, TRUE},
+		{STATUS, "status", "status",
+		 "Show system status\n\n"
+		 "List of status commands (default slot is the currently booted slot):\n"
+		 "  mark-good [booted | other | <SLOT_NAME>] \tMark the slot as good\n"
+		 "  mark-bad [booted | other | <SLOT_NAME>] \tMark the slot as bad\n"
+		 "  mark-active [booted | other | <SLOT_NAME>] \tMark the slot as active",
+		 status_start, status_group, TRUE},
 		{WRITE_SLOT, "write-slot", "write-slot <SLOTNAME> <IMAGE>", "Write image to slot and bypass all update logic", write_slot_start, NULL, FALSE},
 #if ENABLE_SERVICE == 1
 		{SERVICE, "service", "service", "Start RAUC service", service_start, NULL, TRUE},
@@ -1785,24 +1821,18 @@ static void cmdline_handler(int argc, char **argv)
 	RaucCommand *rc;
 	RaucCommand *rcommand = NULL;
 
-	g_option_group_add_entries(install_group, entries_install);
-	g_option_group_add_entries(bundle_group, entries_bundle);
-	g_option_group_add_entries(resign_group, entries_resign);
-	g_option_group_add_entries(convert_group, entries_convert);
-	g_option_group_add_entries(info_group, entries_info);
-	g_option_group_add_entries(status_group, entries_status);
-
 	context = g_option_context_new("<COMMAND>");
 	g_option_context_set_help_enabled(context, FALSE);
 	g_option_context_set_ignore_unknown_options(context, TRUE);
 	g_option_context_add_main_entries(context, entries, NULL);
 	g_option_context_set_description(context,
 			"List of rauc commands:\n"
+#if ENABLE_CREATE == 1
 			"  bundle\tCreate a bundle\n"
 			"  resign\tResign an already signed bundle\n"
-			"  extract\tExtract the bundle content\n"
 			"  convert\tConvert classic to casync bundle\n"
-			"  checksum\tUpdate a manifest with checksums (and optionally sign it)\n"
+#endif
+			"  extract\tExtract the bundle content\n"
 			"  install\tInstall a bundle\n"
 			"  info\t\tShow file information\n"
 #if ENABLE_SERVICE == 1
@@ -1832,7 +1862,7 @@ static void cmdline_handler(int argc, char **argv)
 			g_free(newdomains);
 		}
 		domains = g_getenv("G_MESSAGES_DEBUG");
-		g_message("Debug log domains: '%s'\n", domains);
+		g_message("Debug log domains: '%s'", domains);
 	}
 
 	/* get first parameter without dashes */
@@ -1911,6 +1941,10 @@ static void cmdline_handler(int argc, char **argv)
 			r_context_conf()->keyringpath = keyring;
 		if (signing_keyring)
 			r_context_conf()->signing_keyringpath = signing_keyring;
+		if (mksquashfs_args)
+			r_context_conf()->mksquashfs_args = mksquashfs_args;
+		if (casync_args)
+			r_context_conf()->casync_args = casync_args;
 		if (intermediate)
 			r_context_conf()->intermediatepaths = intermediate;
 		if (mount)
@@ -1951,6 +1985,12 @@ int main(int argc, char **argv)
 	/* disable remote VFS */
 	g_assert(g_setenv("GIO_USE_VFS", "local", TRUE));
 
+	/* Locale needs to support UTF-8, try and flag if unsupported */
+	setlocale(LC_ALL, "");
+	if (g_get_charset(NULL))
+		utf8_supported = TRUE;
+
+	create_option_groups();
 	cmdline_handler(argc, argv);
 
 	return r_exit_status;
