@@ -11,27 +11,51 @@
 #include "context.h"
 #include "signature.h"
 
-/* Define for OpenSSL 1.0.x backwards compatiblity.
- * We use newer get0 names to be clear about memory ownership and to not use
- * API deprecated in OpenSSL 1.1.x */
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L)
-#define X509_get0_notAfter X509_get_notAfter
-#define X509_get0_notBefore X509_get_notBefore
-#endif
-
 GQuark r_signature_error_quark(void)
 {
 	return g_quark_from_static_string("r_signature_error_quark");
 }
 
+/* return 0 for error, 1 for success */
+static int check_purpose_code_sign(const X509_PURPOSE *xp, const X509 *const_x, int ca)
+{
+	/* The external OpenSSL API only takes a non-const X509 pointer, but
+	 * the ex_ variables have already been calculated by other code when
+	 * we are in this callback. */
+	X509 *x = (X509 *)const_x;
+	uint32_t ex_flags = X509_get_extension_flags(x);
+	uint32_t ex_kusage = X509_get_key_usage(x);
+	uint32_t ex_xkusage = X509_get_extended_key_usage(x);
+
+	if (ca) {
+		/* If extended key usage is present, it must contain codeSigning for all
+		 * certs in the chain. */
+		if ((ex_flags & EXFLAG_XKUSAGE) && !(ex_xkusage & XKU_CODE_SIGN)) {
+			g_message("CA certificate extended key usage does not allow code signing");
+			return 0;
+		}
+
+		return X509_check_ca(x);
+	}
+
+	/* If key usage is present, it must contain digitalSignature. */
+	if ((ex_flags & EXFLAG_KUSAGE) && !(ex_kusage & KU_DIGITAL_SIGNATURE)) {
+		g_message("Signer certificate key usage does not allow digital signatures");
+		return 0;
+	}
+
+	/* Extended key usage codeSigning must be present on the leaf. */
+	if (!(ex_flags & EXFLAG_XKUSAGE) || !(ex_xkusage & XKU_CODE_SIGN)) {
+		g_message("Signer certificate does not specify extended key usage code signing");
+		return 0;
+	}
+
+	return 1;
+}
+
 gboolean signature_init(GError **error)
 {
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L)
-	OPENSSL_config(NULL);
-	OpenSSL_add_all_algorithms();
-	ERR_load_crypto_strings();
-#else
-	int ret;
+	int ret, id;
 
 	g_return_val_if_fail(error == FALSE || *error == NULL, FALSE);
 
@@ -50,7 +74,33 @@ gboolean signature_init(GError **error)
 				(flags & ERR_TXT_STRING) ? data : ERR_error_string(err, NULL));
 		return FALSE;
 	}
-#endif
+
+	id = X509_PURPOSE_get_count() + 1;
+	if (X509_PURPOSE_get_by_id(id) >= 0) {
+		g_set_error_literal(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_CRYPTOINIT_FAILED,
+				"Failed to calculate free OpenSSL X509 purpose id");
+		return FALSE;
+	}
+
+	/* X509_TRUST_OBJECT_SIGN maps to the Code Signing ID (via OpenSSL's NID_code_sign) */
+	ret = X509_PURPOSE_add(id, X509_TRUST_OBJECT_SIGN, 0, check_purpose_code_sign, "Code signing", "codesign", NULL);
+	if (!ret) {
+		unsigned long err;
+		const gchar *data;
+		int flags;
+
+		err = ERR_get_error_line_data(NULL, NULL, &data, &flags);
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_CRYPTOINIT_FAILED,
+				"Failed to configure OpenSSL X509 purpose: %s",
+				(flags & ERR_TXT_STRING) ? data : ERR_error_string(err, NULL));
+		return FALSE;
+	}
 
 	return TRUE;
 }
@@ -312,6 +362,97 @@ static GBytes *bytes_from_bio(BIO *bio)
 	return g_bytes_new(data, size);
 }
 
+static gboolean file_contains_crl(const gchar *capath)
+{
+	g_autofree gchar *contents = NULL;
+
+	if (!g_file_test(capath, G_FILE_TEST_IS_REGULAR))
+		return FALSE;
+
+	if (!g_file_get_contents(capath, &contents, NULL, NULL))
+		return FALSE;
+
+	if (strstr(contents, "-----BEGIN X509 CRL-----"))
+		return TRUE;
+
+	return FALSE;
+}
+
+static gboolean contains_crl(const gchar *load_capath, const gchar *load_cadir)
+{
+	if (load_capath && file_contains_crl(load_capath))
+		return TRUE;
+
+	if (load_cadir) {
+		g_autoptr(GDir) dir;
+		const gchar *filename;
+
+		dir = g_dir_open(load_cadir, 0, NULL);
+		if (!dir)
+			return FALSE;
+
+		while ((filename = g_dir_read_name(dir))) {
+			g_autofree gchar *certpath = g_build_filename(load_cadir, filename, NULL);
+
+			if (file_contains_crl(certpath))
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+X509_STORE* setup_x509_store(const gchar *capath, const gchar *cadir, GError **error)
+{
+	const gchar *load_capath = r_context()->config->keyring_path;
+	const gchar *load_cadir = r_context()->config->keyring_directory;
+	const gchar *check_purpose = r_context()->config->keyring_check_purpose;
+	g_autoptr(X509_STORE) store = NULL;
+
+	if (capath)
+		load_capath = strlen(capath) ? capath : NULL;
+	if (cadir)
+		load_cadir = strlen(cadir) ? cadir : NULL;
+
+	if (!(store = X509_STORE_new())) {
+		g_set_error_literal(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_X509_NEW,
+				"failed to allocate new X509 store");
+		return NULL;
+	}
+	if (!X509_STORE_load_locations(store, load_capath, load_cadir)) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_CA_LOAD,
+				"failed to load CA file '%s' and/or directory '%s'", load_capath, load_cadir);
+		return NULL;
+	}
+
+	/* Enable CRL checking if configured */
+	if (r_context()->config->keyring_check_crl)
+		X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL | X509_V_FLAG_EXTENDED_CRL_SUPPORT);
+	else if (contains_crl(load_capath, load_cadir))
+		g_warning("Detected CRL but CRL checking is disabled!");
+
+	/* Enable purpose checking if configured */
+	if (check_purpose) {
+		const X509_PURPOSE *xp = X509_PURPOSE_get0(X509_PURPOSE_get_by_sname(check_purpose));
+		if (!xp || !X509_STORE_set_purpose(store, X509_PURPOSE_get_id(xp))) {
+			g_set_error(
+					error,
+					R_SIGNATURE_ERROR,
+					R_SIGNATURE_ERROR_X509_PURPOSE,
+					"failed to configure X509 purpose '%s'", check_purpose);
+			return NULL;
+		}
+	}
+
+	return g_steal_pointer(&store);
+}
+
 GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, gchar **interfiles, GError **error)
 {
 	GError *ierror = NULL;
@@ -324,7 +465,7 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 	CMS_ContentInfo *cms = NULL;
 	GBytes *res = NULL;
 	int flags = CMS_DETACHED | CMS_BINARY;
-	gchar *keyring_path = NULL;
+	const gchar *keyring_path = NULL, *keyring_dir = NULL;
 
 	g_return_val_if_fail(content != NULL, NULL);
 	g_return_val_if_fail(certfile != NULL, NULL);
@@ -346,7 +487,6 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 	intercerts = sk_X509_new_null();
 
 	for (gchar **intercertpath = interfiles; intercertpath && *intercertpath != NULL; intercertpath++) {
-
 		X509 *intercert = load_cert(*intercertpath, &ierror);
 		if (intercert == NULL) {
 			g_propagate_error(error, ierror);
@@ -392,42 +532,33 @@ GBytes *cms_sign(GBytes *content, const gchar *certfile, const gchar *keyfile, g
 	/* keyring was given, perform verification to obtain trust chain */
 	if (r_context()->signing_keyringpath) {
 		keyring_path = r_context()->signing_keyringpath;
-	} else if (r_context()->config->keyring_path) {
+		keyring_dir = "";
+	} else {
 		keyring_path = r_context()->config->keyring_path;
+		keyring_dir = r_context()->config->keyring_directory;
 	}
 
-	if (keyring_path) {
+	if (keyring_path || keyring_dir) {
 		g_autoptr(CMS_ContentInfo) vcms = NULL;
 		g_autoptr(X509_STORE) store = NULL;
 		STACK_OF(X509) *verified_chain = NULL;
 
-		if (!(store = X509_STORE_new())) {
-			g_set_error_literal(
-					error,
-					R_SIGNATURE_ERROR,
-					R_SIGNATURE_ERROR_X509_NEW,
-					"failed to allocate new X509 store");
-			goto out;
-		}
-		if (!X509_STORE_load_locations(store, keyring_path, NULL)) {
-			g_set_error(
-					error,
-					R_SIGNATURE_ERROR,
-					R_SIGNATURE_ERROR_CA_LOAD,
-					"failed to load CA file '%s'", keyring_path);
+		if (!(store = setup_x509_store(keyring_path, keyring_dir, &ierror))) {
+			g_propagate_error(error, ierror);
+			g_clear_pointer(&res, g_bytes_unref);
 			goto out;
 		}
 
 		g_message("Keyring given, doing signature verification");
 		if (!cms_verify(content, res, store, &vcms, &ierror)) {
 			g_propagate_error(error, ierror);
-			res = NULL;
+			g_clear_pointer(&res, g_bytes_unref);
 			goto out;
 		}
 
 		if (!cms_get_cert_chain(vcms, store, &verified_chain, &ierror)) {
 			g_propagate_error(error, ierror);
-			res = NULL;
+			g_clear_pointer(&res, g_bytes_unref);
 			goto out;
 		}
 
@@ -851,7 +982,7 @@ out:
 	return sig;
 }
 
-gboolean cms_verify_file(const gchar *filename, GBytes *sig, gsize limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error)
+gboolean cms_verify_file(const gchar *filename, GBytes *sig, goffset limit, X509_STORE *store, CMS_ContentInfo **cms, GError **error)
 {
 	GError *ierror = NULL;
 	g_autoptr(GMappedFile) file = NULL;
@@ -871,6 +1002,21 @@ gboolean cms_verify_file(const gchar *filename, GBytes *sig, gsize limit, X509_S
 	}
 	content = g_mapped_file_get_bytes(file);
 
+	/* On 32 bit systems, G_MAXSIZE will be only 32 bit (unsigned) while
+	 * 'limit' is 64 bit (singed). Thus we must take care of not passing
+	 * 'limit' values exceeding G_MAXSIZE to g_bytes_new_from_bytes.
+	 * However, mmapping for large limit values will cause problems,
+	 * anyway.
+	 */
+	if ((guint64)limit > (guint64)G_MAXSIZE) {
+		g_set_error(
+				error,
+				R_SIGNATURE_ERROR,
+				R_SIGNATURE_ERROR_PARSE,
+				"Bundle size exceeds maximum size!");
+		goto out;
+	}
+
 	if (limit) {
 		GBytes *tmp = g_bytes_new_from_bytes(content, 0, limit);
 		g_bytes_unref(content);
@@ -886,4 +1032,3 @@ gboolean cms_verify_file(const gchar *filename, GBytes *sig, gsize limit, X509_S
 out:
 	return res;
 }
-
