@@ -118,6 +118,7 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 {
 	GError *ierror = NULL;
 	gssize size;
+	goffset seeksize;
 	g_autoptr(GFile) srcimagefile = g_file_new_for_path(image->filename);
 	int out_fd = g_unix_output_stream_get_fd(outstream);
 
@@ -132,16 +133,25 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 	g_unix_output_stream_set_close_fd(outstream, FALSE);
 
 	size = g_output_stream_splice((GOutputStream *) outstream, instream,
-			G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+			G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
 			NULL,
 			&ierror);
 	if (size == -1) {
 		g_propagate_prefixed_error(error, ierror,
 				"Failed splicing data: ");
 		return FALSE;
-	} else if (size != (gssize)image->checksum.size) {
+	}
+
+	seeksize = g_seekable_tell((GSeekable*) instream);
+
+	if (seeksize != (goffset)image->checksum.size) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
-				"Written size (%"G_GSIZE_FORMAT ") != image size (%"G_GSIZE_FORMAT ")", size, (gssize)image->checksum.size);
+				"Written size (%"G_GOFFSET_FORMAT ") != image size (%"G_GOFFSET_FORMAT ")", seeksize, image->checksum.size);
+		return FALSE;
+	}
+
+	if (!g_input_stream_close(instream, NULL, &ierror)) {
+		g_propagate_error(error, ierror);
 		return FALSE;
 	}
 
@@ -160,7 +170,7 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, G
 	return TRUE;
 }
 
-static gboolean casync_extract(RaucImage *image, gchar *dest, const gchar *seed, const gchar *store, const gchar *tmpdir, GError **error)
+static gboolean casync_extract(RaucImage *image, gchar *dest, int out_fd, const gchar *seed, const gchar *store, const gchar *tmpdir, GError **error)
 {
 	g_autoptr(GSubprocessLauncher) launcher = NULL;
 	g_autoptr(GSubprocess) sproc = NULL;
@@ -180,10 +190,12 @@ static gboolean casync_extract(RaucImage *image, gchar *dest, const gchar *seed,
 	}
 	g_ptr_array_add(args, g_strdup("--seed-output=no"));
 	g_ptr_array_add(args, g_strdup(image->filename));
-	g_ptr_array_add(args, g_strdup(dest));
+	g_ptr_array_add(args, g_strdup(out_fd >= 0 ? "-" : dest));
 	g_ptr_array_add(args, NULL);
 
 	launcher = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_NONE);
+	if (out_fd >= 0)
+		g_subprocess_launcher_take_stdout_fd(launcher, out_fd);
 	if (tmpdir)
 		g_subprocess_launcher_setenv(launcher, "TMPDIR", tmpdir, TRUE);
 
@@ -229,7 +241,7 @@ static RaucSlot *get_active_slot_class_member(gchar *slotclass)
 	return NULL;
 }
 
-static gboolean casync_extract_image(RaucImage *image, gchar *dest, GError **error)
+static gboolean casync_extract_image(RaucImage *image, gchar *dest, int out_fd, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -265,6 +277,14 @@ static gboolean casync_extract_image(RaucImage *image, gchar *dest, GError **err
 		g_debug("Adding as casync directory tree seed: %s", seedslot->mount_point);
 		seed = g_strdup(seedslot->mount_point);
 	} else {
+		GStatBuf seedstat;
+
+		/* For the moment do not utilize UBI volumes as seed because they are
+		 * character devices - additional logic is needed to (temporarily) map
+		 * them to UBIBLOCK devices which are suitable for that purpose */
+		if (g_stat(seedslot->device, &seedstat) < 0 || S_ISCHR(seedstat.st_mode))
+			goto extract;
+
 		g_debug("Adding as casync blob seed: %s", seedslot->device);
 		seed = g_strdup(seedslot->device);
 	}
@@ -280,7 +300,7 @@ extract:
 		g_debug("Using tmp path: '%s'", tmpdir);
 
 	/* Call casync to extract */
-	res = casync_extract(image, dest, seed, store, tmpdir, &ierror);
+	res = casync_extract(image, dest, out_fd, seed, store, tmpdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -337,12 +357,11 @@ static gboolean write_image_to_dev(RaucImage *image, RaucSlot *slot, GError **er
 		g_message("Extracting %s to %s", image->filename, slot->device);
 
 		/* Extract caibx to device */
-		res = casync_extract_image(image, slot->device, &ierror);
+		res = casync_extract_image(image, slot->device, -1, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
 			goto out;
 		}
-
 	} else {
 		res = copy_raw_image_to_dev(image, slot, &ierror);
 		if (!res) {
@@ -609,9 +628,9 @@ out:
 static gboolean unpack_archive(RaucImage *image, gchar *dest, GError **error)
 {
 	if (g_str_has_suffix(image->filename, ".caidx" ))
-		return casync_extract_image(image, dest, error);
+		return casync_extract_image(image, dest, -1, error);
 	else if (g_str_has_suffix(image->filename, ".catar" ))
-		return casync_extract_image(image, dest, error);
+		return casync_extract_image(image, dest, -1, error);
 	else
 		return untar_image(image, dest, error);
 }
@@ -633,6 +652,7 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 	g_autoptr(GSubprocess) sproc = NULL;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
+	RaucBundle *bundle;
 
 	g_assert_nonnull(slot);
 	g_assert_nonnull(slot->name);
@@ -654,6 +674,8 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 	g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_PARENT", slot->parent ? slot->parent->name : "", TRUE);
 	if (slot->mount_point) {
 		g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_MOUNT_POINT", slot->mount_point, TRUE);
+	} else if (slot->ext_mount_point) {
+		g_subprocess_launcher_setenv(launcher, "RAUC_SLOT_MOUNT_POINT", slot->ext_mount_point, TRUE);
 	}
 	if (image) {
 		g_subprocess_launcher_setenv(launcher, "RAUC_IMAGE_NAME", image->filename, TRUE);
@@ -662,16 +684,19 @@ static gboolean run_slot_hook(const gchar *hook_name, const gchar *hook_cmd, Rau
 	}
 	g_subprocess_launcher_setenv(launcher, "RAUC_MOUNT_PREFIX", r_context()->config->mount_prefix, TRUE);
 
-	if (r_context()->install_info->mounted_bundle) {
+	bundle = r_context()->install_info->mounted_bundle;
+	if (bundle) {
 		gchar **hashes = NULL;
 		gchar *string = NULL;
 
-		hashes = get_pubkey_hashes(r_context()->install_info->mounted_bundle->verified_chain);
+		hashes = get_pubkey_hashes(bundle->verified_chain);
 		string = g_strjoinv(" ", hashes);
 		g_strfreev(hashes);
 
-		g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_SPKI_HASHES", string, FALSE);
+		g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_SPKI_HASHES", string, TRUE);
 		g_free(string);
+
+		g_subprocess_launcher_setenv(launcher, "RAUC_BUNDLE_MOUNT_POINT", bundle->mount_point, TRUE);
 	}
 
 	sproc = g_subprocess_launcher_spawn(
@@ -773,11 +798,22 @@ static gboolean img_to_ubivol_handler(RaucImage *image, RaucSlot *dest_slot, con
 		goto out;
 	}
 
-	/* copy */
-	res = copy_raw_image(image, outstream, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
+	/* Handle casync index file */
+	if (g_str_has_suffix(image->filename, ".caibx")) {
+		g_message("Extracting %s to %s", image->filename, dest_slot->device);
+
+		res = casync_extract_image(image, NULL, out_fd, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	} else {
+		/* copy */
+		res = copy_raw_image(image, outstream, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
 	}
 
 	/* run slot post install hook if enabled */
@@ -825,11 +861,22 @@ static gboolean img_to_ubifs_handler(RaucImage *image, RaucSlot *dest_slot, cons
 		goto out;
 	}
 
-	/* copy */
-	res = copy_raw_image(image, outstream, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto out;
+	/* Handle casync index file */
+	if (g_str_has_suffix(image->filename, ".caibx")) {
+		g_message("Extracting %s to %s", image->filename, dest_slot->device);
+
+		res = casync_extract_image(image, NULL, out_fd, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
+	} else {
+		/* copy */
+		res = copy_raw_image(image, outstream, &ierror);
+		if (!res) {
+			g_propagate_error(error, ierror);
+			goto out;
+		}
 	}
 
 	/* run slot post install hook if enabled */
@@ -1120,7 +1167,7 @@ static gboolean img_to_fs_handler(RaucImage *image, RaucSlot *dest_slot, const g
 	}
 
 	/* run slot post install hook if enabled */
-	if (hook_name && image->hooks.post_install)  {
+	if (hook_name && image->hooks.post_install) {
 		res = mount_and_run_slot_hook(hook_name, R_SLOT_HOOK_POST_INSTALL, dest_slot, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
@@ -1166,9 +1213,9 @@ static gboolean img_to_boot_mbr_switch_handler(RaucImage *image, RaucSlot *dest_
 	g_message("Found inactive boot partition %d (pos. %"G_GUINT64_FORMAT "B, size %"G_GUINT64_FORMAT "B)",
 			inactive_part, dest_partition.start, dest_partition.size);
 
-	if (dest_partition.size < image->checksum.size) {
+	if (dest_partition.size < (guint64)image->checksum.size) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
-				"Size of image (%"G_GSIZE_FORMAT ") does not fit to slot size %"G_GUINT64_FORMAT,
+				"Size of image (%"G_GOFFSET_FORMAT ") does not fit to slot size %"G_GUINT64_FORMAT,
 				image->checksum.size, dest_partition.size);
 		res = FALSE;
 		goto out;
@@ -1249,11 +1296,11 @@ out:
 #if ENABLE_EMMC_BOOT_SUPPORT == 1
 static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
-
 	gboolean res = FALSE;
 	int out_fd;
 	gint part_active;
 	g_autofree gchar *part_active_str = NULL;
+	g_autofree gchar *realdev = NULL;
 	gint part_active_after;
 	g_autoptr(GUnixOutputStream) outstream = NULL;
 	GError *ierror = NULL;
@@ -1273,8 +1320,17 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 		}
 	}
 
+	realdev = r_realpath(dest_slot->device);
+	if (!realdev) {
+		g_set_error(error,
+				R_UPDATE_ERROR,
+				R_UPDATE_ERROR_FAILED,
+				"Can't resolve eMMC device %s", dest_slot->device);
+		goto out;
+	}
+
 	/* read active boot partition from ext_csd */
-	res = r_emmc_read_bootpart(dest_slot->device, &part_active, &ierror);
+	res = r_emmc_read_bootpart(realdev, &part_active, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1284,11 +1340,10 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 			part_active_str = g_strdup("<none>");
 			break;
 		case 6:
-			part_active_str = g_strdup(dest_slot->device);
+			part_active_str = g_strdup(realdev);
 			break;
 		default:
-			part_active_str = g_strdup_printf("%sboot%d", dest_slot->device,
-					part_active);
+			part_active_str = g_strdup_printf("%sboot%d", realdev, part_active);
 	}
 	g_message("Found active eMMC boot partition %s", part_active_str);
 
@@ -1299,7 +1354,7 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 	part_slot = g_new0(RaucSlot, 1);
 	part_slot->device = g_strdup_printf(
 			"%sboot%d",
-			dest_slot->device,
+			realdev,
 			INACTIVE_BOOT_PARTITION(part_active));
 
 	/* disable read-only on determined eMMC boot partition */
@@ -1367,7 +1422,7 @@ static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot,
 	 * Read explicitly from root device (this forces another kernel
 	 * partition switch and should trigger the ext_csd bug more reliably).
 	 */
-	res = r_emmc_read_bootpart(dest_slot->device, &part_active_after, &ierror);
+	res = r_emmc_read_bootpart(realdev, &part_active_after, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1466,11 +1521,12 @@ RaucUpdatePair updatepairs[] = {
 	{"*.ext4.caibx", "ext4", img_to_fs_handler},
 	{"*.ext4.caibx", "raw", img_to_raw_handler},
 	{"*.vfat.caibx", "raw", img_to_raw_handler},
-	//{"*.ubifs.caibx", "ubivol", img_to_ubivol_handler}, /* unsupported */
-	//{"*.ubifs.caibx", "ubifs", img_to_ubifs_handler}, /* unsupported */
+	{"*.ubifs.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.ubifs.caibx", "ubifs", img_to_ubifs_handler},
 	//{"*.img.caibx", "nand", img_to_nand_handler}, /* unsupported */
-	//{"*.img.caibx", "ubivol", img_to_ubivol_handler}, /* unsupported */
-	//{"*.squashfs.caibx", "ubivol", img_to_ubivol_handler}, /* unsupported */
+	{"*.img.caibx", "ubivol", img_to_ubivol_handler},
+	{"*.img.caibx", "ubifs", img_to_ubifs_handler},
+	{"*.squashfs.caibx", "ubivol", img_to_ubivol_handler},
 	{"*.img.caibx", "*", img_to_raw_handler}, /* fallback */
 	{"*.caidx", "ext4", archive_to_ext4_handler},
 	{"*.caidx", "ubifs", archive_to_ubifs_handler},
@@ -1522,7 +1578,7 @@ img_to_slot_handler get_update_handler(RaucImage *mfimage, RaucSlot *dest_slot,
 		}
 	}
 
-	if (handler == NULL)  {
+	if (handler == NULL) {
 		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_NO_HANDLER, "Unsupported image %s for slot type %s",
 				mfimage->filename, dest);
 		goto out;
