@@ -38,6 +38,7 @@ static void config_file_fixture_tear_down(ConfigFileFixture *fixture,
 {
 	g_assert_true(rm_tree(fixture->tmpdir, NULL));
 	g_free(fixture->tmpdir);
+	r_context_clean();
 }
 
 /* Test: Parse entire config file and check if derived slot / file structures
@@ -126,7 +127,7 @@ install-same=false\n";
 	g_assert_true(slot->readonly);
 	g_assert_true(slot->install_same);
 	g_assert_null(slot->parent);
-	g_assert(find_config_slot_by_device(config, "/dev/rescue-0") == slot);
+	g_assert(find_config_slot_by_name(config, "rescue.0") == slot);
 
 	slot = g_hash_table_lookup(config->slots, "rootfs.0");
 	g_assert_cmpstr(slot->name, ==, "rootfs.0");
@@ -137,7 +138,7 @@ install-same=false\n";
 	g_assert_false(slot->readonly);
 	g_assert_false(slot->install_same);
 	g_assert_null(slot->parent);
-	g_assert(find_config_slot_by_device(config, "/dev/rootfs-0") == slot);
+	g_assert(find_config_slot_by_name(config, "rootfs.0") == slot);
 
 	slot = g_hash_table_lookup(config->slots, "rootfs.1");
 	g_assert_cmpstr(slot->name, ==, "rootfs.1");
@@ -148,7 +149,7 @@ install-same=false\n";
 	g_assert_false(slot->readonly);
 	g_assert_false(slot->install_same);
 	g_assert_null(slot->parent);
-	g_assert(find_config_slot_by_device(config, "/dev/rootfs-1") == slot);
+	g_assert(find_config_slot_by_name(config, "rootfs.1") == slot);
 
 	slot = g_hash_table_lookup(config->slots, "appfs.0");
 	g_assert_cmpstr(slot->name, ==, "appfs.0");
@@ -159,7 +160,7 @@ install-same=false\n";
 	g_assert_false(slot->readonly);
 	g_assert_false(slot->install_same);
 	g_assert_nonnull(slot->parent);
-	g_assert(find_config_slot_by_device(config, "/dev/appfs-0") == slot);
+	g_assert(find_config_slot_by_name(config, "appfs.0") == slot);
 
 	slot = g_hash_table_lookup(config->slots, "appfs.1");
 	g_assert_cmpstr(slot->name, ==, "appfs.1");
@@ -170,7 +171,7 @@ install-same=false\n";
 	g_assert_false(slot->readonly);
 	g_assert_false(slot->install_same);
 	g_assert_nonnull(slot->parent);
-	g_assert(find_config_slot_by_device(config, "/dev/appfs-1") == slot);
+	g_assert(find_config_slot_by_name(config, "appfs.1") == slot);
 
 	g_assert_cmpuint(g_list_length(slotlist), ==, 5);
 
@@ -284,14 +285,25 @@ parent=invalid\n\
 	g_clear_error(&ierror);
 }
 
-static void config_file_typo_in_boolean_readonly_key(ConfigFileFixture *fixture,
-		gconstpointer user_data)
+static void config_file_typo(ConfigFileFixture *fixture, const gchar *cfg_file)
 {
 	RaucConfig *config;
 	GError *ierror = NULL;
 	gchar* pathname;
 
-	const gchar *cfg_file = "\
+	pathname = write_tmp_file(fixture->tmpdir, "typo.conf", cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
+	g_assert_null(config);
+	g_clear_error(&ierror);
+}
+
+static void config_file_typo_in_boolean_readonly_key(ConfigFileFixture *fixture,
+		gconstpointer user_data)
+{
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
@@ -302,25 +314,30 @@ description=Rescue partition\n\
 device=/dev/mtd4\n\
 type=raw\n\
 bootname=factory0\n\
-readonly=typo\n";
+readonly=typo\n");
+}
 
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_clear_error(&ierror);
+static void config_file_typo_in_boolean_allow_mounted_key(ConfigFileFixture *fixture,
+		gconstpointer user_data)
+{
+	config_file_typo(fixture, "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+mountprefix=/mnt/myrauc/\n\
+\n\
+[slot.rescue.0]\n\
+description=Rescue partition\n\
+device=/dev/mtd4\n\
+type=raw\n\
+bootname=factory0\n\
+allow-mounted=typo\n");
 }
 
 static void config_file_typo_in_boolean_install_same_key(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
-	gchar* pathname;
-
-	const gchar *cfg_file = "\
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
@@ -331,25 +348,13 @@ description=Rescue partition\n\
 device=/dev/mtd4\n\
 type=raw\n\
 bootname=factory0\n\
-install-same=typo\n";
-
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_clear_error(&ierror);
+install-same=typo\n");
 }
 
 static void config_file_typo_in_boolean_force_install_same_key(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
-	gchar* pathname;
-
-	const gchar *cfg_file = "\
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
@@ -360,25 +365,13 @@ description=Rescue partition\n\
 device=/dev/mtd4\n\
 type=raw\n\
 bootname=factory0\n\
-force-install-same=typo\n";
-
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_clear_error(&ierror);
+force-install-same=typo\n");
 }
 
 static void config_file_typo_in_boolean_ignore_checksum_key(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
-	gchar* pathname;
-
-	const gchar *cfg_file = "\
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
@@ -389,25 +382,13 @@ description=Rescue partition\n\
 device=/dev/mtd4\n\
 type=raw\n\
 bootname=factory0\n\
-ignore-checksum=typo\n";
-
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_clear_error(&ierror);
+ignore-checksum=typo\n");
 }
 
 static void config_file_typo_in_boolean_resize_key(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
-	gchar* pathname;
-
-	const gchar *cfg_file = "\
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
@@ -417,39 +398,18 @@ mountprefix=/mnt/myrauc/\n\
 description=Rescue partition\n\
 device=/dev/null\n\
 type=ext4\n\
-resize=typo\n";
-
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_config.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_clear_error(&ierror);
+resize=typo\n");
 }
 
 static void config_file_typo_in_boolean_activate_installed_key(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
-	gchar* pathname;
-
-	const gchar *cfg_file = "\
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
 mountprefix=/mnt/myrauc/\n\
-activate-installed=typo\n";
-
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_assert_null(config);
-	g_clear_error(&ierror);
+activate-installed=typo\n");
 }
 
 static void config_file_no_max_bundle_download_size(ConfigFileFixture *fixture,
@@ -501,24 +461,11 @@ max-bundle-download-size=0\n";
 static void config_file_typo_in_uint64_max_bundle_download_size(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
-	gchar* pathname;
-
-	const gchar *cfg_file = "\
+	config_file_typo(fixture, "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
 bootloader=barebox\n\
-max-bundle-download-size=no-uint64\n";
-
-	pathname = write_tmp_file(fixture->tmpdir, "invalid_max_bundle_download_size.conf", cfg_file, NULL);
-	g_assert_nonnull(pathname);
-
-	g_assert_false(load_config(pathname, &config, &ierror));
-	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
-	g_assert_null(config);
-
-	g_clear_error(&ierror);
+max-bundle-download-size=no-uint64\n");
 }
 
 static void config_file_activate_installed_set_to_true(ConfigFileFixture *fixture,
@@ -802,7 +749,8 @@ static void config_file_test_write_slot_status(void)
 	r_slot_free_status(ss);
 }
 
-static void config_file_system_serial(void)
+static void config_file_system_serial(ConfigFileFixture *fixture,
+		gconstpointer user_data)
 {
 	g_assert_nonnull(r_context()->system_serial);
 	g_assert_cmpstr(r_context()->system_serial, ==, "1234");
@@ -859,6 +807,54 @@ static void config_file_test_global_slot_status(ConfigFileFixture *fixture,
 	}
 }
 
+static void config_file_keyring_checks(ConfigFileFixture *fixture,
+		gconstpointer user_data)
+{
+	RaucConfig *config;
+	GError *ierror = NULL;
+	gboolean res = FALSE;
+	gchar* pathname;
+
+	const gchar *simple_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+[keyring]\n\
+path=/dev/null\n";
+	const gchar *checking_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+[keyring]\n\
+path=/dev/null\n\
+check-crl=true\n\
+check-purpose=codesign\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "simple.conf", simple_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(config);
+	g_assert_false(config->keyring_check_crl);
+	g_assert_cmpstr(config->keyring_check_purpose, ==, NULL);
+
+	free_config(config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "checking.conf", checking_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	res = load_config(pathname, &config, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+	g_assert_nonnull(config);
+	g_assert_true(config->keyring_check_crl);
+	g_assert_cmpstr(config->keyring_check_purpose, ==, "codesign");
+
+	free_config(config);
+}
+
 int main(int argc, char *argv[])
 {
 	setlocale(LC_ALL, "C");
@@ -877,6 +873,9 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/invalid-parent", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_invalid_parent,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/typo-in-boolean-allow-mounted-key", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_typo_in_boolean_allow_mounted_key,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/typo-in-boolean-readonly-key", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_typo_in_boolean_readonly_key,
 			config_file_fixture_tear_down);
@@ -921,13 +920,18 @@ int main(int argc, char *argv[])
 			config_file_fixture_tear_down);
 	g_test_add_func("/config-file/read-slot-status", config_file_test_read_slot_status);
 	g_test_add_func("/config-file/write-read-slot-status", config_file_test_write_slot_status);
-	g_test_add_func("/config-file/system-serial", config_file_system_serial);
+	g_test_add("/config-file/system-serial", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_system_serial,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/statusfile-missing", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_statusfile_missing,
 			config_file_fixture_tear_down);
 	g_test_add("/config-file/global-slot-staus", ConfigFileFixture, NULL,
 			config_file_fixture_set_up_global, config_file_test_global_slot_status,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/keyring-checks", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_keyring_checks,
+			config_file_fixture_tear_down);
 
 	return g_test_run();
 }
