@@ -27,13 +27,33 @@ static void install_fixture_set_up_bundle(InstallFixture *fixture,
 static void install_fixture_set_up_bundle_central_status(InstallFixture *fixture,
 		gconstpointer user_data)
 {
-
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, "test/test-global.conf");
 	fixture_helper_set_up_bundle(fixture->tmpdir, NULL, FALSE, FALSE);
 }
 
+static void install_fixture_set_up_bundle_already_mounted(InstallFixture *fixture,
+		gconstpointer user_data)
+{
+	const gchar *manifest_file = "\
+[update]\n\
+compatible=Test Config\n\
+\n\
+[hooks]\n\
+filename=hook.sh\n\
+\n\
+[image.bootloader]\n\
+filename=bootloader.ext4\n\
+hooks=install\n\
+";
+
+	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
+
+	fixture_helper_set_up_system(fixture->tmpdir, NULL);
+	fixture_helper_set_up_bundle(fixture->tmpdir, manifest_file, TRUE, TRUE);
+}
+
 static void install_fixture_set_up_bundle_custom_handler(InstallFixture *fixture,
 		gconstpointer user_data)
 {
@@ -206,6 +226,7 @@ static void install_fixture_tear_down(InstallFixture *fixture,
 		return;
 
 	test_umount(fixture->tmpdir, "slot");
+	test_umount(fixture->tmpdir, "bootloader");
 	test_rm_tree(fixture->tmpdir, "");
 }
 
@@ -222,6 +243,7 @@ static void install_test_target(InstallFixture *fixture,
 	GHashTable *tgrp;
 	GList *selected_images = NULL;
 	GError *error = NULL;
+	gboolean result;
 
 
 	const gchar *manifest_file = "\
@@ -259,7 +281,9 @@ filename=bootloader.img";
 
 	r_context_conf()->bootslot = g_strdup("system0");
 
-	g_assert_true(determine_slot_states(NULL));
+	result = determine_slot_states(&error);
+	g_assert_no_error(error);
+	g_assert_true(result);
 
 	g_assert_nonnull(r_context()->config);
 	g_assert_nonnull(r_context()->config->slots);
@@ -299,8 +323,8 @@ filename=bootloader.img";
 	g_assert_cmpint(g_hash_table_size(tgrp), ==, 6);
 
 	selected_images = get_install_images(rm, tgrp, &error);
-	g_assert_nonnull(selected_images);
 	g_assert_no_error(error);
+	g_assert_nonnull(selected_images);
 
 	g_assert_cmpint(g_list_length(selected_images), ==, 4);
 	g_assert_cmpstr(((RaucImage*)g_list_nth_data(selected_images, 0))->filename, ==, "rootfs.ext4");
@@ -624,8 +648,8 @@ device=/dev/null\n\
 	g_assert_nonnull(tgrp);
 
 	selected_images = get_install_images(rm, tgrp, &error);
-	g_assert_nonnull(selected_images);
 	g_assert_no_error(error);
+	g_assert_nonnull(selected_images);
 
 	/* We expecte the image selection to return both appfs.img and
 	 * rootfs.img as we have matching slots for them. */
@@ -841,7 +865,8 @@ device=/dev/null\n\
 	g_assert_no_error(error);
 	g_assert_nonnull(rm);
 
-	install_images = get_install_images(rm, tgrp, NULL);
+	install_images = get_install_images(rm, tgrp, &error);
+	g_assert_no_error(error);
 	g_assert_nonnull(install_images);
 	g_clear_pointer(&rm, free_manifest);
 
@@ -857,7 +882,8 @@ device=/dev/null\n\
 	g_assert_no_error(error);
 	g_assert_nonnull(rm);
 
-	install_images = get_install_images(rm, tgrp, NULL);
+	install_images = get_install_images(rm, tgrp, &error);
+	g_assert_no_error(error);
 	g_assert_nonnull(install_images);
 	g_clear_pointer(&rm, free_manifest);
 
@@ -873,8 +899,9 @@ device=/dev/null\n\
 	g_assert_no_error(error);
 	g_assert_nonnull(rm);
 
-	install_images = get_install_images(rm, tgrp, NULL);
+	install_images = get_install_images(rm, tgrp, &error);
 	g_assert_null(install_images);
+	g_assert_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_FAILED);
 	g_clear_pointer(&rm, free_manifest);
 
 	g_hash_table_unref(tgrp);
@@ -1124,6 +1151,47 @@ static void install_test_bundle_hook_post_install(InstallFixture *fixture,
 	g_free(bundlepath);
 }
 
+/* Test with already mounted slot */
+static void install_test_already_mounted(InstallFixture *fixture,
+		gconstpointer user_data)
+{
+	gchar *bundlepath, *mountprefix, *hookfilepath;
+	RaucInstallArgs *args;
+	GError *ierror = NULL;
+	gboolean res;
+
+	/* needs to run as root */
+	if (!test_running_as_root())
+		return;
+
+	/* Set mount path to current temp dir */
+	mountprefix = g_build_filename(fixture->tmpdir, "mount", NULL);
+	g_assert_nonnull(mountprefix);
+	r_context_conf()->mountprefix = mountprefix;
+	r_context();
+
+	bundlepath = g_build_filename(fixture->tmpdir, "bundle.raucb", NULL);
+	g_assert_nonnull(bundlepath);
+
+	hookfilepath = g_build_filename(fixture->tmpdir, "bootloader", "hook-install-mounted", NULL);
+	g_assert_nonnull(hookfilepath);
+	g_assert_false(g_file_test(hookfilepath, G_FILE_TEST_EXISTS));
+
+	args = install_args_new();
+	args->name = g_strdup(bundlepath);
+	args->notify = install_notify;
+	args->cleanup = install_cleanup;
+	res = do_install_bundle(args, &ierror);
+	g_assert_no_error(ierror);
+	g_assert_true(res);
+
+	g_assert_true(g_file_test(hookfilepath, G_FILE_TEST_IS_REGULAR));
+
+	args->status_result = 0;
+
+	g_free(bundlepath);
+}
+
 static void install_fixture_set_up_system_user(InstallFixture *fixture,
 		gconstpointer user_data)
 {
@@ -1197,5 +1265,9 @@ int main(int argc, char *argv[])
 			install_fixture_set_up_bundle_post_hook, install_test_bundle_hook_post_install,
 			install_fixture_tear_down);
 
+	g_test_add("/install/already-mounted", InstallFixture, NULL,
+			install_fixture_set_up_bundle_already_mounted, install_test_already_mounted,
+			install_fixture_tear_down);
+
 	return g_test_run();
 }
